@@ -46,7 +46,7 @@ from rlbot.utils.class_importer import import_class_with_base, import_agent
 from rlbot.utils.logging_utils import get_logger, DEFAULT_LOGGER
 from rlbot.utils.process_configuration import WrongProcessArgs
 from rlbot.utils.structures.start_match_structures import MAX_PLAYERS
-from rlbot.utils.structures.game_interface import GameInterface
+from rlbot.utils.structures.game_interface import GameInterface, USE_OLD_LAUNCH
 from rlbot.utils.config_parser import mergeTASystemSettings, cleanUpTASystemSettings
 from rlbot.parsing.rlbot_config_parser import LAUNCHER_PREFERENCE_KEY
 from rlbot.matchcomms.server import MatchcommsServerThread
@@ -75,9 +75,11 @@ class ROCKET_LEAGUE_PROCESS_INFO:
 @dataclass
 class RocketLeagueLauncherPreference:
     STEAM = 'steam'
-    EPIC = 'epic'
+    EPIC = 'epic' # This tries epic first, then falls back to steam. Weird name is for backwards compat.
+    EPIC_ONLY = 'epic_only'
     preferred_launcher: str
     use_login_tricks: bool
+    rocket_league_exe_path: Optional[Path] = None
 
 
 # By default, we will attempt Epic with login tricks, then fall back to Steam.
@@ -141,10 +143,7 @@ class SetupManager:
     python_files = None
     bot_bundles: List[BotConfigBundle] = None
     match_config: MatchConfig = None
-    agent_metadata_queue = mp.Queue()
     extension = None
-    bot_processes: Dict[int, BotProcessInfo] = {}
-    script_processes: Dict[int, subprocess.Popen] = {}
 
     def __init__(self):
         self.logger = get_logger(DEFAULT_LOGGER)
@@ -160,6 +159,9 @@ class SetupManager:
         self.matchcomms_server: MatchcommsServerThread = None
         self.early_start_seconds = 0
         self.num_metadata_received = 0
+        self.agent_metadata_queue = mp.Queue()
+        self.bot_processes: Dict[int, BotProcessInfo] = {}
+        self.script_processes: Dict[int, subprocess.Popen] = {}
 
     def is_rocket_league_running(self, port) -> bool:
         """
@@ -205,7 +207,8 @@ class SetupManager:
         elif not self.is_rocket_league_running(port):
             mergeTASystemSettings()
             pref = launcher_preference or self.launcher_preference or DEFAULT_LAUNCHER_PREFERENCE
-            self.launch_rocket_league(port=port, launcher_preference=pref)
+            if not self.launch_rocket_league(port=port, launcher_preference=pref):
+                raise Exception("Failed to launch Rocket League!")
 
         try:
             self.logger.info("Loading interface...")
@@ -228,23 +231,38 @@ class SetupManager:
                 return int(rocket_league_port)
         return None
 
-    def launch_rocket_league(self, port, launcher_preference: RocketLeagueLauncherPreference = DEFAULT_LAUNCHER_PREFERENCE):
+    def launch_rocket_league(self, port, launcher_preference: RocketLeagueLauncherPreference = DEFAULT_LAUNCHER_PREFERENCE) -> bool:
+        if launcher_preference.preferred_launcher == RocketLeagueLauncherPreference.EPIC_ONLY:
+            return self.launch_rocket_league_with_epic(port,  launcher_preference)
+        elif launcher_preference.preferred_launcher == RocketLeagueLauncherPreference.STEAM:
+            return self.launch_rocket_league_with_steam(port)
+        elif launcher_preference.preferred_launcher == RocketLeagueLauncherPreference.EPIC:
+            # Historically, the preference of EPIC has caused RLBot to try Epic first, then Steam.
+            # Keeping that behavior for backwards compatibility.
+            epic_worked = self.launch_rocket_league_with_epic(port, launcher_preference)
+            if epic_worked:
+                return True
+            self.logger.info("Epic launch has failed, falling back to Steam!")
+            return self.launch_rocket_league_with_steam(port)
+    
+    def launch_rocket_league_with_epic(self, port, launcher_preference: RocketLeagueLauncherPreference) -> bool:
         """
         Launches Rocket League but does not connect to it.
         """
         ideal_args = ROCKET_LEAGUE_PROCESS_INFO.get_ideal_args(port)
 
-        if launcher_preference.preferred_launcher == RocketLeagueLauncherPreference.EPIC:
-            if launcher_preference.use_login_tricks:
-                if launch_with_epic_login_trick(ideal_args):
-                    return
-                else:
-                    self.logger.info("Epic login trick seems to have failed!")
-                    # Try Steam after this.
-            if launch_with_epic_simple(ideal_args):
-                return
+        if launcher_preference.use_login_tricks:
+            if launch_with_epic_login_trick(ideal_args, launcher_preference):
+                return True
+            else:
+                self.logger.info("Epic login trick seems to have failed!")
+        if launch_with_epic_simple(ideal_args, launcher_preference):
+            return True
 
+    def launch_rocket_league_with_steam(self, port) -> bool:
         # Try launch via Steam.
+        ideal_args = ROCKET_LEAGUE_PROCESS_INFO.get_ideal_args(port)
+
         steam_exe_path = try_get_steam_executable_path()
         if steam_exe_path:  # Note: This Python 3.8 feature would be useful here https://www.python.org/dev/peps/pep-0572/#abstract
             exe_and_args = [
@@ -254,7 +272,7 @@ class SetupManager:
             ] + ideal_args
             self.logger.info(f'Launching Rocket League with: {exe_and_args}')
             _ = subprocess.Popen(exe_and_args)  # This is deliberately an orphan process.
-            return
+            return True
 
         self.logger.warning(f'Launching Rocket League using Steam-only fall-back launch method with args: {ideal_args}')
         self.logger.info("You should see a confirmation pop-up, if you don't see it then click on Steam! "
@@ -270,7 +288,7 @@ class SetupManager:
 
             try:
                 _ = subprocess.Popen(linux_args)
-                return
+                return True
             except OSError:
                 self.logger.warning('Could not launch Steam executable on Linux.')
 
@@ -280,6 +298,8 @@ class SetupManager:
         except webbrowser.Error:
             self.logger.warning(
                 'Unable to launch Rocket League. Please launch Rocket League manually using the -rlbot option to continue.')
+            return False
+        return True
 
     def load_match_config(self, match_config: MatchConfig, bot_config_overrides={}):
         """
@@ -352,6 +372,10 @@ class SetupManager:
         self.game_interface.match_config = match_config
         self.game_interface.start_match_flatbuffer = match_config.create_flatbuffer()
 
+        if USE_OLD_LAUNCH:
+            self.start_match_configuration = match_config.create_match_settings()
+            self.game_interface.start_match_configuration = self.start_match_configuration
+
     def load_config(self, framework_config: ConfigObject = None, config_location=DEFAULT_RLBOT_CONFIG_LOCATION,
                     bot_configs=None,
                     looks_configs=None):
@@ -454,7 +478,11 @@ class SetupManager:
         # Launch processes
         # TODO: this might be the right moment to fix the player indices based on a game tick packet.
         if not early_starters_only:
-            packet = get_one_packet()
+            if USE_OLD_LAUNCH:
+                packet = game_data_struct.GameTickPacket()
+                self.game_interface.update_live_data_packet(packet)
+            else:
+                packet = get_one_packet()
 
         # TODO: root through the packet and find discrepancies in the player index mapping.
         for i in range(min(self.num_participants, len(match_config.player_configs))):
@@ -474,8 +502,9 @@ class SetupManager:
                 participant_index = None
 
                 self.logger.info(f'Player in slot {i} was sent with spawn id {spawn_id}, will search in the packet.')
-                for n in range(0, packet.PlayersLength()):
-                    packet_spawn_id = packet.Players(n).SpawnId()
+                num_players = packet.num_cars if USE_OLD_LAUNCH else packet.PlayersLength()
+                for n in range(0, num_players):
+                    packet_spawn_id = packet.game_cars[n].spawn_id if USE_OLD_LAUNCH else packet.Players(n).SpawnId()
                     if spawn_id == packet_spawn_id:
                         self.logger.info(f'Looks good, considering participant index to be {n}')
                         participant_index = n
@@ -488,16 +517,19 @@ class SetupManager:
 
             if participant_index not in self.bot_processes:
                 bundle = get_bot_config_bundle(player_config.config_path)
-                name = str(self.match_config.player_configs[i].deduped_name)
+                deduped_name = str(self.match_config.player_configs[i].deduped_name)
                 if bundle.supports_standalone:
                     executable = sys.executable
                     if bundle.use_virtual_environment:
-                        executable = str(Path(bundle.config_directory) / 'venv' / 'Scripts' / 'python.exe')
+                        if platform.system() == "Windows":
+                            executable = str(Path(bundle.config_directory) / 'venv' / 'Scripts' / 'python.exe')
+                        else:
+                            executable = str(Path(bundle.config_directory) / 'venv' / 'bin' / 'python')
                     process = subprocess.Popen([
                         executable,
                         bundle.python_file,
                         '--config-file', str(player_config.config_path),
-                        '--name', name,
+                        '--name', deduped_name,
                         '--team', str(self.teams[i]),
                         '--player-index', str(participant_index),
                         '--spawn-id', str(spawn_id),
@@ -506,7 +538,7 @@ class SetupManager:
                     self.bot_processes[participant_index] = BotProcessInfo(process=None, subprocess=process, player_config=player_config)
 
                     # Insert immediately into the agent metadata map because the standalone process has no way to communicate it back out
-                    self.agent_metadata_map[participant_index] = AgentMetadata(participant_index, name, self.teams[i], {process.pid})
+                    self.agent_metadata_map[participant_index] = AgentMetadata(participant_index, deduped_name, self.teams[i], {process.pid})
                 else:
                     reload_request = mp.Event()
                     quit_callback = mp.Event()
@@ -514,7 +546,7 @@ class SetupManager:
                     self.bot_quit_callbacks.append(quit_callback)
                     process = mp.Process(target=SetupManager.run_agent,
                                          args=(self.quit_event, quit_callback, reload_request, self.bot_bundles[i],
-                                               name,
+                                               deduped_name,
                                                self.teams[i], participant_index, self.python_files[i], self.agent_metadata_queue,
                                                match_config, self.matchcomms_server.root_url, spawn_id))
                     process.start()
@@ -532,7 +564,10 @@ class SetupManager:
                 continue
             executable = sys.executable
             if script_config_bundle.use_virtual_environment:
-                executable = str(Path(script_config_bundle.config_directory) / 'venv' / 'Scripts' / 'python.exe')
+                if platform.system() == "Windows":
+                    executable = str(Path(script_config_bundle.config_directory) / 'venv' / 'Scripts' / 'python.exe')
+                else:
+                    executable = str(Path(script_config_bundle.config_directory) / 'venv' / 'bin' / 'python')
 
             process = subprocess.Popen(
                 [
@@ -632,12 +667,23 @@ class SetupManager:
             try:
                 single_agent_metadata: AgentMetadata = self.agent_metadata_queue.get(timeout=0.1)
                 num_recieved += 1
+                if single_agent_metadata.name not in [pc.deduped_name for pc in self.match_config.player_configs]:
+                    self.logger.warn(f"Got agent metadata for {single_agent_metadata.name} but it shouldn't be running!")
+
                 self.helper_process_manager.start_or_update_helper_process(single_agent_metadata)
                 self.agent_metadata_map[single_agent_metadata.index] = single_agent_metadata
                 process_configuration.configure_processes(self.agent_metadata_map, self.logger)
             except queue.Empty:
                 self.num_metadata_received += num_recieved
-                return num_recieved
+                break
+
+        # Let's go through the agent metadata map and see if we can expand it with any child processes.
+        # We'll do it every time this function is called (generall periodically),
+        # we don't know when an agent might spawn another process.
+        if process_configuration.append_child_pids(self.agent_metadata_map):
+            process_configuration.configure_processes(self.agent_metadata_map, self.logger)
+
+        return num_recieved
 
     def reload_all_agents(self, quiet=False):
         if not quiet:
@@ -645,7 +691,7 @@ class SetupManager:
         for rr in self.bot_reload_requests:
             rr.set()
 
-    def shut_down(self, time_limit=5, kill_all_pids=False, quiet=False):
+    def shut_down(self, time_limit=5, kill_all_pids=True, quiet=False):
         if not quiet:
             self.logger.info("Shutting Down")
 
@@ -679,6 +725,14 @@ class SetupManager:
 
         self.kill_matchcomms_server()
 
+        # Drain the agent_metadata_queue to make sure nothing rears its head later.
+        while True:  # will exit on queue.Empty
+            try:
+                metadata = self.agent_metadata_queue.get(timeout=0.1)
+                self.logger.warn(f"Drained out metadata for {metadata.name} during shutdown!")
+            except queue.Empty:
+                break
+
         # The quit event can only be set once. Let's reset to our initial state
         self.quit_event = mp.Event()
         self.helper_process_manager = HelperProcessManager(self.quit_event)
@@ -727,36 +781,38 @@ class SetupManager:
         self.bot_processes.clear()
         self.num_metadata_received = 0
 
+    def send_sigterm_recursive(self, pid: int) -> List[psutil.Process]:
+        """
+        Returns the list of processes under the pid and including the pid for further handling,
+        because they may become orphaned by the sigterm.
+        """
+        all_processes = []
+        immediate_children = []
+        try:
+            process = psutil.Process(pid)
+            immediate_children = [c for c in process.children(recursive=False)]
+            all_processes = [process] + [c for c in process.children(recursive=True)]
+            process.send_signal(signal.SIGTERM)
+        except Exception as ex:
+            self.logger.debug(f"Got {ex} while sending sigterm to pid {pid}.")
+
+        for c in immediate_children:
+            self.send_sigterm_recursive(c.pid)
+
+        return all_processes
+
     def kill_agent_process_ids(self, pids: Set[int]):
+        all_processes = []
         for pid in pids:
-            try:
-                process = psutil.Process(pid)
-                process.send_signal(signal.SIGTERM)
-            except Exception as ex:
-                self.logger.warn(f"Got {ex} while terminating pid {pid}.")
+            all_processes += self.send_sigterm_recursive(pid)
 
         time.sleep(.5)
 
-        for pid in pids:
+        for c in all_processes:
             try:
-                parent = psutil.Process(pid)
-                for child in parent.children(recursive=True):  # or parent.children() for recursive=False
-                    self.logger.info(f"Killing {child.pid} (child of {pid})")
-                    try:
-                        child.kill()
-                    except psutil.NoSuchProcess:
-                        self.logger.info("Already dead.")
-                self.logger.info(f"Killing {pid}")
-                try:
-                    parent.kill()
-                except psutil.NoSuchProcess:
-                    self.logger.info("Already dead.")
-            except psutil.NoSuchProcess:
-                self.logger.info("Can't fetch parent process, already dead.")
-            except psutil.AccessDenied as ex:
-                self.logger.error(f"Access denied when trying to kill a bot pid! {ex}")
+                c.kill()
             except Exception as ex:
-                self.logger.error(f"Unexpected exception when trying to kill a bot pid! {ex}")
+                self.logger.debug(f"Got {ex} while killing pid {pid}.")
 
     def kill_matchcomms_server(self):
         if self.matchcomms_server:
