@@ -190,7 +190,7 @@ def run_exercises(setup_manager: SetupManager, exercises: Iterable[Exercise], se
                 time.sleep(0.03)
 
                 update_row('>>>>', ren.renderman.white)
-                result = _grade_exercise(game_interface, exercise, seed)
+                result = _grade_exercise(setup_manager, exercise, seed)
 
                 if isinstance(result.grade, Pass):
                     update_row('PASS', ren.renderman.green)
@@ -266,10 +266,11 @@ def _setup_exercise(game_interface: GameInterface, ex: Exercise, seed: int) -> O
     game_interface.set_game_state(game_state)
 
 
-def _grade_exercise(game_interface: GameInterface, ex: Exercise, seed: int) -> Result:
+def _grade_exercise(setup_manager: SetupManager, ex: Exercise, seed: int) -> Result:
     grade = None
     rate_limit = rate_limiter.RateLimiter(120)
-    last_tick_game_time = None  # What the tick time of the last observed tick was
+    metadata_ticker = rate_limiter.RateLimiter(2)
+    game_interface = setup_manager.game_interface
     game_tick_packet = GameTickPacket()  # We want to do a deep copy for game inputs so people don't mess with em
 
     # Run until the Exercise finishes.
@@ -278,15 +279,16 @@ def _grade_exercise(game_interface: GameInterface, ex: Exercise, seed: int) -> R
         # Read from game data shared memory
         game_interface.update_live_data_packet(game_tick_packet)
 
-        # Run ex.on_tick() only if the game_info has updated.
-        tick_game_time = game_tick_packet.game_info.seconds_elapsed
-        if tick_game_time != last_tick_game_time:
-            last_tick_game_time = tick_game_time
-            try:
-                grade = ex.on_tick(game_tick_packet)
-                ex.render(game_interface.renderer)
-            except Exception as e:
-                return Result(ex, seed, FailDueToExerciseException(e, traceback.format_exc()))
+        try:
+            grade = ex.on_tick(game_tick_packet)
+            ex.render(game_interface.renderer)
+            if metadata_ticker.acquire_if_ready():
+                # Check for agent metadata regularly. Ensures bots are properly started,
+                # constrained to certain CPU cores, and run at high process priority.
+                # This mimics what RLBotGUI does during a match.
+                setup_manager.try_recieve_agent_metadata()
+        except Exception as e:
+            return Result(ex, seed, FailDueToExerciseException(e, traceback.format_exc()))
 
         rate_limit.acquire()
 
