@@ -4,9 +4,31 @@
 # 3) we can import it into your module module
 # https://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package
 
-__version__ = '1.62.6'
+__version__ = '1.67.0'
 
 release_notes = {
+    '1.67.0': """
+    Moving the items.csv file into RLBot so it can be used by both GUI and bots. - VirxEC
+    """,
+    '1.66.1': """
+    Fix launching bots with venvs on non-windows. - VirxEC
+    New epic_only launcher preference, and accepting Rocket League location option. - tarehart
+    Bumping flatbuffers version. - VirxEC
+    """,
+    '1.65.1': """
+    When shutting down, kill all processes by default instead of opt-in. - tarehart
+    Improvement to Epic launch. - VirxEC
+    Fix Linux launch. - VirxEC
+    Make RLBot's python virtual enviroment system more efficient. - VirxEC
+    """,
+    '1.64.0': """
+    Warning in performance readout when processes are fighting over a bot index. - tarehart
+    """,
+    '1.63.2': """
+    Killing processes more effectively at the end of matches. - tarehart
+    Preventing bot startup commands from leaking into the next match. - tarehart
+    Processing agent metadata regularly during training exercises. - tarehart
+    """,
     '1.62.6': """
     Updating strategy for launching Epic version of Rocket League. - tarehart
     Fixing race conditions in the Epic launch process. - SavageSnowgoose
